==> gentrix-0.1.1.crate: gentrix-0.1.1/src/matrix.rs <==
#[derive(Debug)]
pub struct Matrix<T: Clone> {
    matrix: Vec<Vec<Option<T>>>,
}

impl<T: Clone> Matrix<T> {
    pub fn new(size: (usize, usize)) -> Matrix<T> {
        let (m, n) = size;
        let mut matrix: Vec<Vec<Option<T>>> = Vec::new();

        let row = vec![None; n];

        for _ in 0..m {
            matrix.push(row.clone());
        }

        return Matrix::<T> { matrix };
    }

    pub fn get(&mut self, pos: (usize, usize)) -> Option<T> {
        let (m, n) = pos;

        let item = self.matrix[m][n].clone();

        if item.is_some() {
            self.clear(pos);
        }

        return item;
    }

    pub fn get_ref(&self, pos: (usize, usize)) -> Option<&T> {
        let (m, n) = pos;

        return self.matrix[m][n].as_ref();
    }

    pub fn get_mut(&mut self, pos: (usize, usize)) -> Option<&mut T> {
        let (m, n) = pos;

        return self.matrix[m][n].as_mut();
    }

    pub fn set(&mut self, item: T, pos: (usize, usize)) {
        let (m, n) = pos;
        self.matrix[m][n] = Some(item);
    }

    pub fn clear(&mut self, pos: (usize, usize)) {
        let (m, n) = pos;

        self.matrix[m][n] = None;
    }

    pub fn translate(&mut self, from: (usize, usize), dir: (usize, usize)) -> Result<(), ()> {
        let item = self.get(from);

        if item.is_none() {
            return Err(());
        }

        self.set(item.unwrap(), dir);
        self.clear(from);

        return Ok(());
    }
}

impl<T: Clone + Copy> From<Vec<Vec<T>>> for Matrix<T> {
    fn from(items: Vec<Vec<T>>) -> Self {
        let m = items.len();
        let n = items.first().unwrap().len();
        let mut matrix = Matrix::new((m, n));

        for i in 0..m {
            for j in 0..n {
                matrix.set(items[i][j], (i, j));
            }
        }

        return matrix;
    }
}

==> gentrix-0.1.2.crate: gentrix-0.1.2/src/matrix.rs <==
#[derive(Debug)]
pub struct Matrix<T: Clone> {
    matrix: Vec<Vec<Option<T>>>,
}

impl<T: Clone> Matrix<T> {
    pub fn new(size: (usize, usize)) -> Matrix<T> {
        let (m, n) = size;
        let mut matrix: Vec<Vec<Option<T>>> = Vec::new();

        let row = vec![None; n];

        for _ in 0..m {
            matrix.push(row.clone());
        }

        return Matrix::<T> { matrix };
    }

    pub fn get(&mut self, pos: (usize, usize)) -> Option<T> {
        let (m, n) = pos;

        let item = self.matrix[m][n].clone();

        if item.is_some() {
            self.clear(pos);
        }

        return item;
    }

    pub fn get_ref(&self, pos: (usize, usize)) -> Option<&T> {
        let (m, n) = pos;

        return self.matrix[m][n].as_ref();
    }

    pub fn get_mut(&mut self, pos: (usize, usize)) -> Option<&mut T> {
        let (m, n) = pos;

        return self.matrix[m][n].as_mut();
    }

    pub fn set(&mut self, item: T, pos: (usize, usize)) {
        let (m, n) = pos;
        self.matrix[m][n] = Some(item);
    }

    pub fn clear(&mut self, pos: (usize, usize)) {
        let (m, n) = pos;

        self.matrix[m][n] = None;
    }

    pub fn translate(&mut self, from: (usize, usize), dir: (isize, isize)) -> Result<(), ()> {
        let item = self.get(from);

        if item.is_none() {
            return Err(());
        }

        let to = (
            (from.0 as isize + dir.0) as usize,
            (from.1 as isize + dir.1) as usize,
        );

        self.set(item.unwrap(), to);
        self.clear(from);

        return Ok(());
    }
}

impl<T: Clone + Copy> From<Vec<Vec<T>>> for Matrix<T> {
    fn from(items: Vec<Vec<T>>) -> Self {
        let m = items.len();
        let n = items.first().unwrap().len();
        let mut matrix = Matrix::new((m, n));

        for i in 0..m {
            for j in 0..n {
                matrix.set(items[i][j], (i, j));
            }
        }

        return matrix;
    }
}
